package cat.hack3.codingtests.marsrover;



import java.util.logging.Logger;

/**
 * Notice that the representation is the following:
 * Latitude is Y, starts from 1 on the top and goes down incrementally
 * Longitude is X, starts from 1 on the left and goes to right incrementally
 *     1    2    3
 * 1  1-1  1-2  1-3
 * 2  2-1  2-2  2-3
 * 3  3-1  3-2  3-3
 */
public class Coordinates {
    private final int latitude;
    private final int longitude;
    private final Logger logger = Logger.getLogger(Coordinates.class.getName());

    public static Coordinates of(int latitude, int longitude) {
        return new Coordinates(latitude, longitude);
    }

    public Coordinates(int latitude, int longitude) {
        this.latitude = latitude;
        this.longitude = longitude;
    }

    public Coordinates getNextPositionTowards(Direction direction) {
        var newLatitude = latitude;
        var newLongitude = longitude;
        switch (direction) {
            case NORTH -> newLatitude--;
            case SOUTH -> newLatitude++;
            case WEST -> newLongitude--;
            case EAST -> newLongitude++;
        }
        logger.info(String.format("Updated coordinates towards %s, now is %d-%d",
                direction, newLatitude, newLongitude));

        return new Coordinates(newLatitude, newLongitude);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Coordinates that = (Coordinates) o;

        if (latitude != that.latitude) return false;
        return longitude == that.longitude;
    }

    @Override
    public String toString() {
        return "Coordinates{" +
                "latitude=" + latitude +
                ", longitude=" + longitude +
                '}';
    }

}
